@@ -2,10 +2,10 @@ package autobox::Transform;
 
 use strict;
 use warnings;
-use v5.10;
+use 5.010;
 use parent qw/autobox/;
 
-our $VERSION = "1.012";
+our $VERSION = "1.013";
 
 =head1 NAME
 
@@ -13,11 +13,11 @@ autobox::Transform - Autobox methods to transform Arrays and Hashes
 
 =head1 SYNOPSIS
 
-=head2 Examples
-
     use autobox::Core;  # uniq, sort, join, sum, etc.
     use autobox::Transform;
 
+=head2 Array Examples
+
     $books->map_by("genre");
     $books->map_by(price_with_tax => [$tax_pct]);
 
@@ -26,11 +26,54 @@ autobox::Transform - Autobox methods to transform Arrays and Hashes
 
     $books->uniq_by("id");
 
-    $books->group_by("title");
-    $books->group_by_count("genre")
-    $books->group_by_array("genre")
+    $books->group_by("title"),
+    # {
+    #     "Leviathan Wakes"       => $books->[0],
+    #     "Caliban's War"         => $books->[1],
+    #     "The Tree-Body Problem" => $books->[2],
+    #     "The Name of the Wind"  => $books->[3],
+    # },
 
+    $authors->group_by(publisher_affiliation => ["with"]),
+    # {
+    #     'James A. Corey with Orbit'     => $authors->[0],
+    #     'Cixin Liu with Head of Zeus'   => $authors->[1],
+    #     'Patrick Rothfuss with Gollanz' => $authors->[2],
+    # },
+
+    $books->group_by_count("genre"),
+    # {
+    #     "Sci-fi"  => 3,
+    #     "Fantasy" => 1,
+    # },
+
+    my $genre_books = $books->group_by_array("genre");
+    # {
+    #     "Sci-fi"  => [ $sf_book_1, $sf_book_2, $sf_book_3 ],
+    #     "Fantasy" => [ $fantasy_book_1 ],
+    # },
+
+    $authors->map_by("books") # ->books returns an arrayref
+    # [ [ $book1, $book2 ], [ $book3 ] ]
     $authors->map_by("books")->flat;
+    # [ $book1, $book2, $book3 ]
+
+
+=head2 Hash Examples
+
+    # Upper-case the genre name, and make the count say "n books"
+    $genre_count->map_each(sub { uc( $_[0] ) => "$_ books" });
+    # {
+    #     "FANTASY" => "1 books",
+    #     "SCI-FI"  => "3 books",
+    # },
+
+    # Transform each pair to the string "n: genre"
+    $genre_count->map_each_to_array(sub { "$_: $_[0]" });
+    # [ "1: Fantasy", "3: Sci-fi" ]
+
+
+=head2 Combined examples
 
     my $order_authors = $order->books
         ->uniq_by("isbn")
@@ -45,6 +88,9 @@ autobox::Transform - Autobox methods to transform Arrays and Hashes
 
 =head2 Comparison of vanilla Perl and autobox version
 
+These are only for when there's a straightforward and simple Perl
+equivalent.
+
     ### map_by - method call: $books are Book objects
     my @genres = map { $_->genre() } @$books;
     my @genres = $books->map_by("genre");
@@ -87,42 +133,6 @@ autobox::Transform - Autobox methods to transform Arrays and Hashes
     my $distinct_books = $books->uniq_by("id");
 
 
-
-    ### group_by - method call: $books are Book objects
-
-    $books->group_by("title"),
-    # {
-    #     "Leviathan Wakes"       => $books->[0],
-    #     "Caliban's War"         => $books->[1],
-    #     "The Tree-Body Problem" => $books->[2],
-    #     "The Name of the Wind"  => $books->[3],
-    # },
-
-    $authors->group_by(publisher_affiliation => ["with"]),
-    # {
-    #     'James A. Corey with Orbit'     => $authors->[0],
-    #     'Cixin Liu with Head of Zeus'   => $authors->[1],
-    #     'Patrick Rothfuss with Gollanz' => $authors->[2],
-    # },
-
-    $books->group_by_count("genre"),
-    # {
-    #     "Sci-fi"  => 3,
-    #     "Fantasy" => 1,
-    # },
-
-    my $genre_books = $books->group_by_array("genre");
-    # {
-    #     "Sci-fi"  => [ $sf_book_1, $sf_book_2, $sf_book_3 ],
-    #     "Fantasy" => [ $fantasy_book_1 ],
-    # },
-
-
-    ### group_by - hash key: $books are book hashrefs
-    $books->group_by("title"), # $books are hashrefs
-
-
-
     #### flat - $author->books returns an arrayref of Books
     my $author_books = [ map { @{$_->books} } @$authors ]
     my $author_books = $authors->map_by("books")->flat
@@ -167,6 +177,19 @@ $array->flat()
 =back
 
 
+=over 4
+
+=item
+
+$hash->map_each
+
+=item
+
+$hash->map_each_to_array
+
+=back
+
+
 =head2 Raison d'etre
 
 L<autobox::Core> is awesome, for a variety of reasons.
@@ -662,6 +685,26 @@ sub key_value_if_defined {
 
 
 
+=head2 map_each($key_value_subref) : %new_hash | %$new_hash
+
+Map each key-value pair in the hash using the
+$key_value_subref. Similar to how to how map transforms a list into
+another list, map_each transforms a hash into another hash.
+
+C<$key_value_subref->($key, $value)> is called for each pair (with $_
+set to the value).
+
+The subref should return an even-numbered list with zero or more
+key-value pairs which will make up the %new_hash. Typically two items
+are returned in the list (the key and the value).
+
+=head3 Example
+
+    { a => 1, b => 2 }->map_each(sub { "$_[0]$_[0]" => $_ * 2 });
+    # Returns { aa => 2, bb => 4 }
+
+=cut
+
 sub map_each {
     my $hash = shift;
     my ($key_value_subref) = @_;
@@ -675,8 +718,8 @@ sub map_each {
             {
                 local $_ = $value;
                 my (@new_key_value) = $key_value_subref->($key, $value);
-                @new_key_value >= 3 and Carp::croak("map_each \$key_value_subref returned more than the new key and value");
-                ( @new_key_value[ 0..1 ]);
+                (@new_key_value % 2) and Carp::croak("map_each \$key_value_subref returned odd number of keys/values");
+                @new_key_value;
             }
         }
         keys %$hash,
@@ -685,6 +728,24 @@ sub map_each {
     return wantarray ? %$new_hash : $new_hash;
 }
 
+=head2 map_each_to_array($item_subref) : %new_array | %$new_array
+
+Map each key-value pair in the hash into a list using the
+$item_subref.
+
+C<$item_subref->($key, $value)> is called for each pair (with $_ set
+to the value) in key order.
+
+The subref should return zero or more list items which will make up
+the %new_array. Typically one item is returned.
+
+=head3 Example
+
+    { a => 1, b => 2 }->map_each_to_array(sub { "$_[0]-$_" });
+    # Returns [ "a-1", "b-2" ]
+
+=cut
+
 sub map_each_to_array {
     my $hash = shift;
     my ($array_item_subref) = @_;
